#include "B4RDefines.h"

namespace B4R
{
    void B4RAdafruit_TCS34725::Initialize()
    {
       rcs = new (backend) Adafruit_TCS34725();
    };
    void B4RAdafruit_TCS34725::Initialize1(byte TCS34725_INTEGRATIONTIME,byte TCS34725_GAIN)
    {
       rcs = new (backend) Adafruit_TCS34725((uint8_t) TCS34725_INTEGRATIONTIME,(tcs34725Gain_t) TCS34725_GAIN);
    };
    
#if defined (ESP32)         
    bool B4RAdafruit_TCS34725::beginESP32(byte deviceAddr, byte I2Cbus, int sda, int scl)
    {      
      if (I2Cbus==2) {TwoWire Wire2=TwoWire(2); Wire2.begin(sda, scl); return rcs->begin(deviceAddr, &Wire2);}
      else if (I2Cbus==1) {TwoWire Wire1=TwoWire(1); Wire1.begin(sda, scl); return rcs->begin(deviceAddr, &Wire1);}
      else {TwoWire Wire0=TwoWire(0); Wire0.begin(sda, scl); return rcs->begin(deviceAddr, &Wire0);}
     }; 
#endif          
#if defined (ESP8266)
    bool B4RAdafruit_TCS34725::beginESP8266(byte deviceAddr, int sda, int scl)
    {
       TwoWire Wire0=TwoWire(); Wire0.begin(sda, scl); return rcs->begin(deviceAddr, &Wire0);
    };   
#endif

    bool B4RAdafruit_TCS34725::begin(byte deviceAddr)
    { 
      return rcs-> begin(deviceAddr);
    };

    void B4RAdafruit_TCS34725::setIntegrationTime(byte it)
    {
       rcs->setIntegrationTime(it);
    };

    void B4RAdafruit_TCS34725::setGain(byte gain)
    {
       rcs->setGain((tcs34725Gain_t)gain);
    };

    void B4RAdafruit_TCS34725::getRawData(ArrayUInt* rgbc)
    {  uint16_t* RGBC =  (uint16_t*)rgbc->data;
       rcs->getRawData(&RGBC[0], &RGBC[1], &RGBC[2], &RGBC[3]);
    };

    void B4RAdafruit_TCS34725::getRGB(ArrayDouble* rgb)
    {  float* RGB = (float*) rgb->data;
       rcs->getRGB(&RGB[0], &RGB[1], &RGB[2]);
    };

    void B4RAdafruit_TCS34725::getRawDataOneShot(ArrayUInt* rgbc)
    {  uint16_t* RGBC = (uint16_t*) rgbc->data;
       rcs->getRawDataOneShot(&RGBC[0], &RGBC[1], &RGBC[2], &RGBC[3]);
    };

    uint16_t B4RAdafruit_TCS34725::calculateColorTemperature(ArrayUInt* rgb)
    {  uint16_t* RGB = (uint16_t*)rgb->data;
       return rcs->calculateColorTemperature(RGB[0], RGB[1], RGB[2]);
    };

    uint16_t B4RAdafruit_TCS34725::calculateColorTemperature_dn40(ArrayUInt* rgbc)
    {  uint16_t* RGBC = (uint16_t*) rgbc->data;
       return rcs->calculateColorTemperature_dn40(RGBC[0], RGBC[1], RGBC[2], RGBC[3]);
    };

    uint16_t B4RAdafruit_TCS34725::calculateLux(ArrayUInt* rgb)
    {  uint16_t* RGB = (uint16_t*) rgb->data;
       return rcs->calculateLux(RGB[0], RGB[1], RGB[2]);
    };

    void B4RAdafruit_TCS34725::write8(byte reg, byte value)
    {
       rcs->write8(reg, value);
    };

    byte B4RAdafruit_TCS34725::read8(byte reg)
    {
       return rcs->read8(reg);
    };

    uint16_t B4RAdafruit_TCS34725::read16(byte reg)
    {
       return rcs->read16(reg);
    };

    void B4RAdafruit_TCS34725::setInterrupt(bool flag)
    {
       rcs->setInterrupt(flag);
    };

    void B4RAdafruit_TCS34725::clearInterrupt()
    {
       rcs->clearInterrupt();
    };

    void B4RAdafruit_TCS34725::setIntLimits(uint16_t l, uint16_t h)
    {
       rcs->setIntLimits(l, h);
    };

    void B4RAdafruit_TCS34725::enable()
    {
       rcs->enable();
    };

    void B4RAdafruit_TCS34725::disable()
    {
       rcs->disable();
    };
//?? /** TCS34725_INTEGRATIONTIME setting **/
//?? #define /*byte TCS34725_INTEGRATIONTIME_2_4MS;*/ B4R_TCS34725_INTEGRATIONTIME_2_4MS   1
//?? #define /*byte TCS34725_INTEGRATIONTIME_24MS;*/ B4R_TCS34725_INTEGRATIONTIME_24MS     2
//?? #define /*byte TCS34725_INTEGRATIONTIME_50MS;*/ B4R_TCS34725_INTEGRATIONTIME_50MS     3
//?? #define /*byte TCS34725_INTEGRATIONTIME_60MS;*/ B4R_TCS34725_INTEGRATIONTIME_60MS     4
//?? #define /*byte TCS34725_INTEGRATIONTIME_101MS:*/ B4R_TCS34725_INTEGRATIONTIME_101MS   5
//?? #define /*byte TCS34725_INTEGRATIONTIME_120MS;*/ B4R_TCS34725_INTEGRATIONTIME_120MS   6
//?? #define /*byte TCS34725_INTEGRATIONTIME_154MS;*/ B4R_TCS34725_INTEGRATIONTIME_154MS   7
//?? #define /*byte TCS34725_INTEGRATIONTIME_180MS;*/ B4R_TCS34725_INTEGRATIONTIME_180MS   8
//?? #define /*byte TCS34725_INTEGRATIONTIME_199MS;*/ B4R_TCS34725_INTEGRATIONTIME_199MS   9
//?? #define /*byte TCS34725_INTEGRATIONTIME_240MS;*/ B4R_TCS34725_INTEGRATIONTIME_240MS   10
//?? #define /*byte TCS34725_INTEGRATIONTIME_300MS;*/ B4R_TCS34725_INTEGRATIONTIME_300MS   11
//?? #define /*byte TCS34725_INTEGRATIONTIME_360MS;*/ B4R_TCS34725_INTEGRATIONTIME_360MS   12
//?? #define /*byte TCS34725_INTEGRATIONTIME_401MS;*/ B4R_TCS34725_INTEGRATIONTIME_401MS   13
//?? #define /*byte TCS34725_INTEGRATIONTIME_420MS;*/ B4R_TCS34725_INTEGRATIONTIME_420MS   14
//?? #define /*byte TCS34725_INTEGRATIONTIME_480MS;*/ B4R_TCS34725_INTEGRATIONTIME_480MS   15
//?? #define /*byte TCS34725_INTEGRATIONTIME_499MS;*/ B4R_TCS34725_INTEGRATIONTIME_499MS   16
//?? #define /*byte TCS34725_INTEGRATIONTIME_540MS;*/ B4R_TCS34725_INTEGRATIONTIME_540MS   17
//?? #define /*byte TCS34725_INTEGRATIONTIME_600MS;*/ B4R_TCS34725_INTEGRATIONTIME_600MS   18
//?? #define /*byte TCS34725_INTEGRATIONTIME_614MS;*/ B4R_TCS34725_INTEGRATIONTIME_614MS   19          
//?? /** Gain settings for TCS34725  */
//?? #define /*byte   TCS34725_GAIN_1X;*/ B4R_TCS34725_GAIN_1X    0x00
//?? #define /*byte   TCS34725_GAIN_4X;*/ B4R_TCS34725_GAIN_4X    0x01
//?? #define /*byte   TCS34725_GAIN_16X;*/ B4R_TCS34725_GAIN_16X  0x02
//?? #define /*byte   TCS34725_GAIN_60X;*/ B4R_TCS34725_GAIN_60X  0x03
//?? /** Wire definition for TCS34725*/
//?? #define /*byte  I2C_TwoWire0;*/   B4R_I2C_TwoWire0              0x0;
//?? #define /*byte  I2C_TwoWire1;*/   B4R_I2C_TwoWire1              0x1;
//?? #define /*byte  I2C_TwoWire2;*/   B4R_I2C_TwoWire2              0x2;    
//?? #define /*byte  I2C_Wire;*/       I2C_Wire                      0xFF;    

}
