#pragma once
#include "B4RDefines.h"
#include "Adafruit_TCS34725.h"

    //~Event: 
    //~Version: 1.0 
    //~Author - 
    //~Libray from Arduino

namespace B4R {
    //~shortname:Adafruit_TCS34725
    class B4RAdafruit_TCS34725
    {
        private:
            uint8_t backend[sizeof(Adafruit_TCS34725)];
            Adafruit_TCS34725* rcs;
        public: 
            /**
            * Constructor with values by default
            */
            void Initialize();
            /**
            * Constructor with optional values
            */
            void Initialize1(byte TCS34725_INTEGRATIONTIME,byte TCS34725_GAIN);
#if defined (ESP32)
            /**  
            * Initializes I2C and configures pins for esp32 with Wire0, Wire1, Wire2
            * sda by default = -1 scl by default = -1
            * return True if initialization was successful, otherwise false.
            */                                 
            bool beginESP32(byte deviceAddr, byte I2Cbus, int sda, int scl);  
#endif   
#if defined (ESP8266)
            /**  
            * Initializes default I2C and configures with pins for esp8266
            * sda by default = -1 scl by default = -1
            * return True if initialization was successful, otherwise false.
            */
            bool beginESP8266(byte deviceAddr, int sda, int scl);  
#endif      
            /**
            * Initializes default I2C with default pins 
            * return True if initialization was successful, otherwise false. 
            */
            bool begin(byte deviceAddr);
            /**  
            * Sets the integration time for the TC34725 
            */
            void setIntegrationTime(byte it);
            /**
            * Adjusts the gain on the TCS34725
            */
            void setGain(byte TCS34725_GAIN);
            /**
            * Reads the raw red, green, blue and clear channel values
            * first Dim rgbc(4) as UInt / and rgbc(0)=r, rgbc(1)=g, rgbc(2)=b, rgbc(3)=c
            */
            void getRawData(ArrayUInt* rgbc);
            /**
            * Read the RGB color detected by the sensor.
            * first Dim rgb(3) as UInt / and rgbc(0)=r, rgbc(1)=g, rgbc(2)=b
            */
            void getRGB(ArrayDouble* rgb);
            /**
            * Reads the raw red, green, blue and clear channel values in one-shot mode 
            * (e.g., wakes from sleep, takes measurement, enters sleep)
            * first Dim rgbc(4) as UInt / and rgbc(0)=r, rgbc(1)=g, rgbc(2)=b, rgbc(3)=c
            */
            void getRawDataOneShot(ArrayUInt* rgbc);
            /**
            * Converts the raw R/G/B values to color temperature in degrees Kelvin
            * first Dim rgb(3) as UInt / and rgbc(0)=r, rgbc(1)=g, rgbc(2)=b
            */
            uint16_t calculateColorTemperature(ArrayUInt* rgb);
            /**
            * Converts the raw R/G/B values to color temperature in degrees Kelvin using the algorithm described in DN40 from Taos (now AMS).
            * first Dim rgbc(4) as UInt / and rgbc(0)=r, rgbc(1)=g, rgbc(2)=b, rgbc(3)=c
            */
            uint16_t calculateColorTemperature_dn40(ArrayUInt* rgbc);
            /**
            * Converts the raw R/G/B values to lux
            * first Dim rgb(3) as UInt / and rgbc(0)=r, rgbc(1)=g, rgbc(2)=b
            */
            uint16_t calculateLux(ArrayUInt* rgb);
            /**
            * Writes a register and an 8 bit value over I2C
            */
            void write8(byte reg, byte value);
            /**
            * Reads an 8 bit value over I2C
            */
            byte read8(byte reg);
            /**
            * Reads a 16 bit values over I2C
            */
            uint16_t read16(byte reg);
            /**
            * Sets interrupt for TCS34725
            */
            void setInterrupt(bool flag);
            /**
            * Clears inerrupt for TCS34725
            */
            void clearInterrupt();
            /**
            * Sets interrupt limits
            */
            void setIntLimits(uint16_t l, uint16_t h);
            /**
            * Enables the device
            */
            void enable();
            /**
            * Disables the device (putting it in lower power sleep mode)
            */
            void disable();

          // * TCS34725_INTEGRATIONTIME setting **/
            #define /*byte TCS34725_INTEGRATIONTIME_2_4MS;*/ B4R_TCS34725_INTEGRATIONTIME_2_4MS   0xFF
            #define /*byte TCS34725_INTEGRATIONTIME_24MS;*/  B4R_TCS34725_INTEGRATIONTIME_24MS    0xF6
            #define /*byte TCS34725_INTEGRATIONTIME_50MS;*/  B4R_TCS34725_INTEGRATIONTIME_50MS    0xEB
            #define /*byte TCS34725_INTEGRATIONTIME_60MS;*/  B4R_TCS34725_INTEGRATIONTIME_60MS    0xE7
            #define /*byte TCS34725_INTEGRATIONTIME_101MS:*/ B4R_TCS34725_INTEGRATIONTIME_101MS   0xD6
            #define /*byte TCS34725_INTEGRATIONTIME_120MS;*/ B4R_TCS34725_INTEGRATIONTIME_120MS   0xCE
            #define /*byte TCS34725_INTEGRATIONTIME_154MS;*/ B4R_TCS34725_INTEGRATIONTIME_154MS   0xC0
            #define /*byte TCS34725_INTEGRATIONTIME_180MS;*/ B4R_TCS34725_INTEGRATIONTIME_180MS   0xB5
            #define /*byte TCS34725_INTEGRATIONTIME_199MS;*/ B4R_TCS34725_INTEGRATIONTIME_199MS   0xAD
            #define /*byte TCS34725_INTEGRATIONTIME_240MS;*/ B4R_TCS34725_INTEGRATIONTIME_240MS   0x9C
            #define /*byte TCS34725_INTEGRATIONTIME_300MS;*/ B4R_TCS34725_INTEGRATIONTIME_300MS   0x83
            #define /*byte TCS34725_INTEGRATIONTIME_360MS;*/ B4R_TCS34725_INTEGRATIONTIME_360MS   0x6A
            #define /*byte TCS34725_INTEGRATIONTIME_401MS;*/ B4R_TCS34725_INTEGRATIONTIME_401MS   0x59
            #define /*byte TCS34725_INTEGRATIONTIME_420MS;*/ B4R_TCS34725_INTEGRATIONTIME_420MS   0x51
            #define /*byte TCS34725_INTEGRATIONTIME_480MS;*/ B4R_TCS34725_INTEGRATIONTIME_480MS   0x38
            #define /*byte TCS34725_INTEGRATIONTIME_499MS;*/ B4R_TCS34725_INTEGRATIONTIME_499MS   0x30
            #define /*byte TCS34725_INTEGRATIONTIME_540MS;*/ B4R_TCS34725_INTEGRATIONTIME_540MS   0x1F
            #define /*byte TCS34725_INTEGRATIONTIME_600MS;*/ B4R_TCS34725_INTEGRATIONTIME_600MS   0x06
            #define /*byte TCS34725_INTEGRATIONTIME_614MS;*/ B4R_TCS34725_INTEGRATIONTIME_614MS   0x00          
          // * Gain settings for TCS34725  */
            #define /*byte  TCS34725_GAIN_1X;*/  B4R_TCS34725_GAIN_1X           0x00
            #define /*byte  TCS34725_GAIN_4X;*/  B4R_TCS34725_GAIN_4X           0x01
            #define /*byte  TCS34725_GAIN_16X;*/ B4R_TCS34725_GAIN_16X          0x02
            #define /*byte  TCS34725_GAIN_60X;*/ B4R_TCS34725_GAIN_60X          0x03
          // * Wire definition for TCS34725*/
            #define /*byte  I2C_TwoWire0;*/       B4R_I2C_TwoWire0              0x0
            #define /*byte  I2C_TwoWire1;*/       B4R_I2C_TwoWire1              0x1
            #define /*byte  I2C_TwoWire2;*/       B4R_I2C_TwoWire2              0x2   
    };
}
